#!/usr/bin/env python
import jwt
import json, sys, time
from cryptography.hazmat.backends import default_backend
from collections import OrderedDict

def sign_poet(payload, private_key_string, issuer, expires=63072000):
    
    
    #Set the headers
    headers= OrderedDict()
    headers["typ"]="JWT"
    headers["alg"]="RS256"
    
    # Add/overwrite payload
    payload["iss"] = issuer
    payload["iat"] = int(time.time())
    
    
    #Set to expire (two years is default)
    payload["exp"] = payload["iat"] + expires
    jwt_encoded = jwt.encode(payload, private_key_string, algorithm='RS256', headers=headers)
    # Adding a decode in the return. Otherwise returns b'bytestring
    return jwt_encoded.decode('utf-8')

#Command line app.
if __name__ == "__main__":


    if len(sys.argv) not in (4,5):
        print("Usage:")
        print("sign_poet.py [PAYLOAD_JSON_FILE] [SIGNING_PRIVATE_PEM_CERTIFICATE_FILE] [ISSUER] [SECONDS_UNTIL_EXPIRY]")
        print("Example: sign_poet.py mypayload.json my-certificate.pem example.org 31536000")
        print("Note: 31536000 is one year from now.")
        sys.exit(1)

    my_payload_file         = sys.argv[1]
    my_private_key_file     = sys.argv[2]
    issuer                  = sys.argv[3]
    if sys.argv==5:
        expires             = sys.argv[4]
    else:
        expires = 63072000
    my_payload_fh = open(my_payload_file)
    my_private_key_fh = open(my_private_key_file)
        
    #convert json to dict
    try:
        j = my_payload_fh.read()
    
        
        my_payload = json.loads(j)
        if type(my_payload) ==type({}):
            result = sign_poet(my_payload, my_private_key_fh.read(), issuer, expires)
        else:
            result = "JSON was not an object {}"
    except ValueError:
        result = ["Error parsing JSON", str(sys.exc_info())]
        
    
    my_payload_fh.close()        
    my_private_key_fh.close()
    
    print(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

"""
pypoetri
FILE: sample_payload_poet
Created: 5/4/16 8:58 AM


"""
__author__ = 'Mark Scrimshire:@ekivemark'

import sys, json, time

from collections import OrderedDict

def sample_payload_poet(mode="info"):
    """ create a skeleton jwt based on poet

     https://github.com/HHSIDEALab/poet

     Header

    {
    "alg": "RS256",
    "typ": "JWT"
    }

    Payload

    {
    "iss": "nate-trust.org",
    "iat": 1455031265,
    "exp": 1549639265,
    "aud": "apps-dstu2.smarthealthit.org",
    "sub": "jrocket@apps-dstu2.smarthealthit.org",
    "certification_uid": "9292010131",
       "contacts" : [
          "info@smartplatforms.org",
          "https://gallery.smarthealthit.org"
       ],
      "client_name" : "Cardiac Risk App",
      "client_uri": "https://apps-dstu2.smarthealthit.org/cardiac-risk/",
       "logo_uri" : "https://gallery.smarthealthit.org/img/apps/66.png",
       "initiate_login_uri" : "https://apps-dstu2.smarthealthit.org/cardiac-risk/launch.html",
       "redirect_uris" : [
          "https://apps-dstu2.smarthealthit.org/cardiac-risk/"
       ],
       "scope" : "openid profile patient/*.read",
       "token_endpoint_auth_method" : "none",
       "grant_types" : [ "authorization_code" ]
        }
    Signature

    HMACSHA256(
      base64UrlEncode(header) + "." +
      base64UrlEncode(payload),

    ) secret base64 encoded


     """

    my_mode = mode
    output = ""
    expires = 63072000

    if my_mode.lower() == "info":
        info = "\n#jwt Info===========================\n"
        info += "A signed jwt (pronounced 'jot' contains 2 parts:\n"
        info += "1. Header\n"
        info += "2. Payload file\n"
        info += "These are shown below. You can cut and paste the sections to create\n"
        info += "your own payload file.\n"

        output += info

    if my_mode.lower() in ['info', 'both', 'header']:
        header = OrderedDict()

        header['alg'] = "RS256"
        header["typ"] = "JWT"

        if my_mode.lower() in ['info', 'both']:
            output += "\n#jwt Header Settings================\n"
        output += json.dumps(header, indent=4)

        if my_mode.lower() in ['info', 'both']:
            output += "\n\n#===================================\n"

    if my_mode.lower() in ['info', 'both', 'payload']:
        payload = OrderedDict()

        payload['iss'] = "trust-issuer.org"
        payload['iat'] = int(time.time())
        # Set to expire (two years is default)
        payload["exp"] = payload["iat"] + expires
        payload["aud"] = "app.example.com",
        payload["sub"] = "your_admin_email@example.com",
        payload["certification_uid"] = "Unique_Id_Number",
        payload["contacts"] = [
                                  "info@example.com",
                                  "https://help.app.example.com"
                              ],
        payload["client_name"] = "example app name",
        payload["client_uri"] = "https://apps.example.com/example_url/",
        payload["logo_uri"] = "https://apps.example.com/img/logo.png",
        payload["initiate_login_uri"] = "https://apps.example.com/example_app/launch.html",
        payload["redirect_uris"] = [
                                       "https://apps.example.com/example_app/"
                                   ],
        payload["scope"] = "openid profile patient/*.read",
        payload["token_endpoint_auth_method"] = "none",
        payload["grant_types"] = ["authorization_code"]

        if my_mode.lower() in ['info', 'both']:
            output += "\n#jwt Payload Settings===============\n"

        output += json.dumps(payload, indent=4)

        if my_mode.lower() in ['info', 'both']:
            output += "\n#iat = time now. \n"
            output += "#exp = expire time after iat (two years is default).\n"

            output += "\n\n#===================================\n"



    return output

#Command line app.
if __name__ == "__main__":


    if len(sys.argv) < 2:
        print("Usage: sample_payload_poet.py header|payload|both|info")
        print("Example: sample_payload_poet.py info")

        sys.exit(1)

    my_mode                 = sys.argv[1]

    output = ""
    expires = 63072000

    if my_mode.lower() == "info":

        info =  "\n#jwt Info===========================\n"
        info += "A signed jwt (pronounced 'jot' contains 2 parts:\n"
        info += "1. Header\n"
        info += "2. Payload file\n"
        info += "These are shown below. You can cut and paste the sections to create\n"
        info += "your own payload file.\n"

        output += info

    if my_mode.lower() in ['info', 'both', 'header']:
        header = OrderedDict()

        header['alg'] = "RS256"
        header["typ"] = "JWT"

        if my_mode.lower() in ['info', 'both']:
            output += "\n#jwt Header Settings================\n"
        output += json.dumps(header, indent=4)

        if my_mode.lower() in ['info', 'both']:
            output += "\n\n#===================================\n"

    if my_mode.lower() in ['info', 'both', 'payload']:
        payload = OrderedDict()

        payload['iss'] = "trust-issuer.org"
        payload['iat'] = int(time.time())
        # Set to expire (two years is default)
        payload["exp"] = payload["iat"] + expires
        payload["aud"] =  "app.example.com",
        payload["sub"] = "your_admin_email@example.com",
        payload["certification_uid"] = "Unique_Id_Number",
        payload["contacts"] = [
                        "info@example.com",
                        "https://help.app.example.com"
                    ],
        payload["client_name"] = "example app name",
        payload["client_uri"] = "https://apps.example.com/example_url/",
        payload["logo_uri"] = "https://apps.example.com/img/logo.png",
        payload["initiate_login_uri"] = "https://apps.example.com/example_app/launch.html",
        payload["redirect_uris"] = [
                         "https://apps.example.com/example_app/"
                     ],
        payload["scope"] = "openid profile patient/*.read",
        payload["token_endpoint_auth_method"] = "none",
        payload["grant_types"] = ["authorization_code"]

        if my_mode.lower() in ['info', 'both']:

            output += "\n#jwt Payload Settings===============\n"

        output += json.dumps(payload, indent=4)

        if my_mode.lower() in ['info', 'both']:

            output += "\n#iat = time now. \n"
            output += "#exp = expire time after iat (two years is default).\n"

            output += "\n\n#===================================\n"

    print(output)




#!/usr/bin/env python
import jwt
import sys
from cryptography.x509 import load_pem_x509_certificate, OID_COMMON_NAME
from cryptography.hazmat.backends import default_backend
from collections import OrderedDict

def verify_poet(jws, public_key_string):
    python_version = sys.version_info.major
    #print("jws:", jws)
    #print("Public_key_string:", public_key_string)
    jws = jws.rstrip().lstrip()
    
    #print(sys.version_info)

    certBytes = list(public_key_string.encode())
    
    if python_version == 3:    
        certBytes = bytes(certBytes)
    
    certificate = load_pem_x509_certificate(certBytes, default_backend())
    publicKey = certificate.public_key()
    
    cn = certificate.subject.get_attributes_for_oid(OID_COMMON_NAME)[0].value
    try:
        payload = jwt.decode(jws, publicKey, algorithms=['RS256'])
    except jwt.exceptions.ExpiredSignatureError:
        payload = """{"error":"EXPIRED"}"""
    
    if payload.get('iss', ''):
        if payload['iss']!=cn:
            payload = """{"error":"The CN (Common Name) in the public certificate did not match the iss (Issuer) in the JWT payload."}"""    
    else:
        payload = """{"error":"iss (Issuer) was not found in the payload."}"""
    return payload

#command line app.
if __name__ == "__main__":

    if len(sys.argv)!=3:
        print("Usage:")
        print("verify_poet.py [ENCODED_JWT_FILE] [PUBLIC_CERT_FILE]")
        print("Example: verify_poet.py my.jwt my_public_cert.pem")
        sys.exit(1)
    
    jwt_path = sys.argv[1]
    public_key_path = sys.argv[2]
    
    jwt_fh = open(jwt_path)
    public_key_fh = open(public_key_path)
    jws = jwt_fh.read()
    payload = verify_poet(jws, public_key_fh.read())

    print(payload)

    public_key_fh.close()
    jwt_fh.close()



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(name="poetri",
      version="0.0.1",
      description="Pre-OAuth Entity Trust Reference Implementation",
      long_description="""Command line tools and libraries for signing and verifying JWTs according to https://github.com/hhsidealab/poet.""",
      author="Alan Viars",
      author_email="sales@videntity.com",
      url="https://github.com/transparenthealth/poetri",
      download_url="https://github.com/transparenthealth/poetri/tarball/master",
      install_requires=['PyJWT', 'cryptography', ],
      packages= ['poetri', 'tests'],
      package_data={'':['taxonomy-license-crosswalk.csv','fiftythousand.csv']},
      scripts=['poetri/verify_poet.py',
               'poetri/sign_poet.py',
               'poetri/sample_payload_poet.py',
               ]
      )

